%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%            Inverzni iterace s shiftovanim: konvergence
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Priklad 1: mocninna metoda zakladni

A=tst_1;                         % dominantni vl. cislo je jednoznacne 4
[V,D]=eig(A); 

x0=randn(7,1);
[ X,L ] = mocn( A,x0,20 );       % mocninna met. konverguje rychle

subplot(2,1,1)
plot(L,'*')                      % aproximace vl. c.
subplot(2,1,2)                   % chyba aproximace
semilogy(abs(L-4*ones(1,21)),'*')
pause

% zopakuj experiment s A=tst_3; (ma dvojnasobne dom. vl. c. 4) 
% -> konverguje rychle

% zopakuj experiment s A=tst_4; (dom. vl. c. 4, pak dalsi 3.9999) 
% -> chova se jako s dvojnasobnym vl. c.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Priklad 2: mocn. metoda a divergence

A=tst_7;                         % dominantni vl. cislo neni jednoznacne 4, -4

x0=zeros(7,1); x0(1)=1.5;        % startovni vektor pro mocninnou metodu
[ X,L ] = mocn( A,x0,20 );       

subplot(2,1,1)
plot(L,'*')                      % konverguje, ale ne k dom. vl. cislu!
subplot(2,1,2)                   % chyba aproximace
semilogy(abs(L-4*ones(1,21)),'*')
pause

% prohledni sloupce X -> sloupce diverguji

%% zmenime startovni vektor - jak to ovlivni konvergenci?

[V,D]=eig(A);                    
v1=V(:,1);                       % vlastni vektor k cislu -4
v2=V(:,2);                       % vlastni vektor k cislu 4

[ X,L ] = mocn( A,(v1+0.01*rand(7,1)), 20 );
plot(L,'*')                      % konverguje k cislu -4
pause

[ X,L ] = mocn( A,(v2+0.01*rand(7,1)),20 );
plot(L,'*')                      % konverguje k cislu 4
pause

% experimentujte se startovnim vektorem a sledujte chovani metody


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Priklad 3: inv. metoda s fixnim shiftem 
%
% nyni pouzijeme inverzni metodu s fixnim shiftem

A=tst_7;                         % dominantni vl. cislo neni jednoznacne 4, -4

x0=randn(7,1);                   % startovni vektor pro mocninnou metodu
[ X,L ] = invers( A,x0,20, 3.8 );

subplot(2,1,1)
plot(L,'*')                      % konverguje k vl. cislu 4
subplot(2,1,2)                   % chyba aproximace
semilogy(abs(L-4*ones(1,21)),'*')
pause

x0=randn(7,1);                   % startovni vektor pro mocninnou metodu
[ X,L ] = invers( A,x0,20, -4.5 );

subplot(2,1,1)
plot(L,'*')                      % konverguje k vl. cislu -4
subplot(2,1,2)                   % chyba aproximace
semilogy(abs(L+4*ones(1,21)),'*')
pause

% experimentujte se shiftem, sledujte chovani metody


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Priklad 4: inv. metoda s Reyleighovym shiftem

A=tst_4;                         % dominantni vl. cislo je jednoznacne 4,
                                 % ale nasledujici vl. cislo je mu blizke 3.9999

x0=zeros(7,1); x0(1)=1.5;    
                   
[ X,L ] = invers_Reyl( A,x0,20 ); % konverguje velmi rychle k 4 -> matice
                                  % (A-shift*I) je numericky singularni

subplot(2,1,1)
plot(L,'*')                      % konverguje k vl. cislu -4
subplot(2,1,2)                   % chyba aproximace
semilogy(abs(L-4*ones(1,21)),'*')
pause                  
                 
% mente startovni vektor a sledujte, kam metoda konverguje 

x0=zeros(7,1); x0(3)=1;          % konverguje k 1.5 - overte


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Priklad 5: degenerativni pripad divergence inv. metody s Reyl. shiftem

A=[0 1; 1 0];

x0=[0;1];                               
[ Xr,Lr ] = invers_Reyl( A,x0,20 ); % prohlednete si sloupce X -> diverguje
plot(Lr, 'go')                    % Reyl. podil je stale nulovy
pause

x0=[1;2];
[ Xr,Lr ] = invers_Reyl( A,x0,5 ); 
plot(Lr, 'go')                    % konverguje k vl. cislu 1
pause                             % sloupce X konverguji k vl. vektoru

x0=[-1;2];
[ Xr,Lr ] = invers_Reyl( A,x0,5 ); 
plot(Lr, 'go')                    % konverguje k vl. cislu -1
                                  % sloupce X konverguji k vl. vektoru   
